@@ -298,7 +298,7 @@ fun _execute_operation(
   my ($fields) = $type->_collect_fields(
     $context,
     $operation->{selections},
-    {},
+    [[], {}],
     {},
   );
   DEBUG and _debug('_execute_operation(fields)', $fields, $root_value);
@@ -323,13 +323,14 @@ fun _execute_fields(
   (InstanceOf['GraphQL::Type']) $parent_type,
   Any $root_value,
   ArrayRef $path,
-  Map[StrNameValid,ArrayRef[HashRef]] $fields,
+  FieldsGot $fields,
 ) :ReturnType(ExecutionPartialResult | Promise) {
   my (%name2executionresult, @errors);
   my $promise_present;
   DEBUG and _debug('_execute_fields', $parent_type->to_string, $fields, $root_value);
-  for my $result_name (keys %$fields) { # TODO ordering of fields
-    my $nodes = $fields->{$result_name};
+  my ($field_names, $nodes_defs) = @$fields;
+  for my $result_name (@$field_names) {
+    my $nodes = $nodes_defs->{$result_name};
     my $field_node = $nodes->[0];
     my $field_name = $field_node->{name};
     my $field_def = _get_field_def($context->{schema}, $parent_type, $field_name);
@@ -412,7 +413,7 @@ fun _execute_fields_serially(
   (InstanceOf['GraphQL::Type']) $parent_type,
   Any $root_value,
   ArrayRef $path,
-  Map[StrNameValid,ArrayRef[HashRef]] $fields,
+  FieldsGot $fields,
 ) {
   DEBUG and _debug('_execute_fields_serially', $parent_type->to_string, $fields, $root_value);
   # TODO implement
