@@ -7,7 +7,7 @@ use Type::Library
   -base,
   -declare => qw(
     StrNameValid FieldMapInput ValuesMatchTypes DocumentLocation JSONable
-    ErrorResult
+    ErrorResult FieldsGot
   );
 use Type::Utils -all;
 use Types::TypeTiny -all;
@@ -82,6 +82,18 @@ EOF
     };
 };
 
+=head2 FieldsGot
+
+Data item describing the fields found in a particular object in a query.
+Preserves their order.
+
+=cut
+
+declare "FieldsGot", as Tuple[ArrayRef[StrNameValid], Map[
+  StrNameValid,
+  ArrayRef[HashRef]
+]];
+
 =head2 FieldMapInput
 
 Hash-ref mapping field names to a hash-ref
